# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: skytorch/torch/server/service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'skytorch/torch/server/service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#skytorch/torch/server/service.proto\x12\x0eskytorch.torch\"*\n\x14\x44\x65leteTensorsRequest\x12\x12\n\ntensor_ids\x18\x01 \x03(\x04\"\xcc\x01\n\x0eTensorMetadata\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\r\n\x05shape\x18\x02 \x03(\x03\x12\r\n\x05\x64type\x18\x03 \x01(\t\x12\x0e\n\x06nbytes\x18\x04 \x01(\x03\x12\x13\n\x0b\x64\x65vice_type\x18\x05 \x01(\t\x12\x0e\n\x06stride\x18\x06 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x07 \x01(\x03\x12\x14\n\x0c\x64\x65vice_index\x18\x08 \x01(\x05\x12\x17\n\ntensor_ref\x18\t \x01(\x04H\x00\x88\x01\x01\x42\r\n\x0b_tensor_ref\"\xe7\x01\n\x0bTensorChunk\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\x14\n\x0c\x63hunk_number\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x14\n\x0ctotal_chunks\x18\x04 \x01(\r\x12\r\n\x05shape\x18\x05 \x03(\x03\x12\x0e\n\x06stride\x18\x06 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x07 \x01(\x03\x12\r\n\x05\x64type\x18\x08 \x01(\t\x12\x13\n\x0btotal_bytes\x18\t \x01(\x04\x12\x30\n\x08metadata\x18\n \x01(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"2\n\x0eTensorResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"^\n\x10GetScalarRequest\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\x37\n\x0ftensor_metadata\x18\x02 \x03(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"^\n\x11GetScalarResponse\x12\x15\n\x0b\x66loat_value\x18\x01 \x01(\x01H\x00\x12\x13\n\tint_value\x18\x02 \x01(\x03H\x00\x12\x14\n\nbool_value\x18\x03 \x01(\x08H\x00\x42\x07\n\x05value\"\xd1\x01\n\x13\x43reateTensorRequest\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\r\n\x05shape\x18\x02 \x03(\x03\x12\r\n\x05\x64type\x18\x03 \x01(\t\x12\x0e\n\x06nbytes\x18\x04 \x01(\x03\x12\x13\n\x0b\x64\x65vice_type\x18\x05 \x01(\t\x12\x0e\n\x06stride\x18\x06 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x07 \x01(\x03\x12\x14\n\x0c\x64\x65vice_index\x18\x08 \x01(\x05\x12\x17\n\ntensor_ref\x18\t \x01(\x04H\x00\x88\x01\x01\x42\r\n\x0b_tensor_ref\"\x9d\x01\n\x10GetTensorRequest\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\r\n\x05shape\x18\x02 \x03(\x03\x12\r\n\x05\x64type\x18\x03 \x01(\t\x12\x0e\n\x06stride\x18\x04 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x05 \x01(\x03\x12\x30\n\x08metadata\x18\x06 \x01(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"\xe8\x01\n\x11\x43opyTensorRequest\x12\x15\n\rsrc_tensor_id\x18\x01 \x01(\x04\x12\x15\n\rdst_tensor_id\x18\x02 \x01(\x04\x12\x12\n\nsrc_offset\x18\x03 \x01(\x03\x12\x12\n\ndst_offset\x18\x04 \x01(\x03\x12\x11\n\tnum_bytes\x18\x05 \x01(\x03\x12\x34\n\x0csrc_metadata\x18\x06 \x01(\x0b\x32\x1e.skytorch.torch.TensorMetadata\x12\x34\n\x0c\x64st_metadata\x18\x07 \x01(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"$\n\x0fTensorReference\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\"\xc7\x02\n\x0c\x41tenArgument\x12\x31\n\x06tensor\x18\x01 \x01(\x0b\x32\x1f.skytorch.torch.TensorReferenceH\x00\x12\x16\n\x0cscalar_float\x18\x02 \x01(\x01H\x00\x12\x14\n\nscalar_int\x18\x03 \x01(\x03H\x00\x12\x15\n\x0bscalar_bool\x18\x04 \x01(\x08H\x00\x12\x17\n\rscalar_string\x18\x05 \x01(\tH\x00\x12\x36\n\nlist_value\x18\x06 \x01(\x0b\x32 .skytorch.torch.AtenArgumentListH\x00\x12\x14\n\nnone_value\x18\x07 \x01(\x08H\x00\x12\x16\n\x0cscalar_dtype\x18\x08 \x01(\tH\x00\x12\x1e\n\x14scalar_memory_format\x18\t \x01(\tH\x00\x12\x17\n\rscalar_layout\x18\n \x01(\tH\x00\x42\x07\n\x05value\"R\n\x10\x41tenArgumentList\x12,\n\x06values\x18\x01 \x03(\x0b\x32\x1c.skytorch.torch.AtenArgument\x12\x10\n\x08is_tuple\x18\x02 \x01(\x08\"\x82\x03\n\x12\x45xecuteAtenRequest\x12\x0f\n\x07op_name\x18\x01 \x01(\t\x12*\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1c.skytorch.torch.AtenArgument\x12\x30\n\x07outputs\x18\x03 \x03(\x0b\x32\x1f.skytorch.torch.TensorReference\x12>\n\x06kwargs\x18\x04 \x03(\x0b\x32..skytorch.torch.ExecuteAtenRequest.KwargsEntry\x12\x37\n\x0ftensor_metadata\x18\x05 \x03(\x0b\x32\x1e.skytorch.torch.TensorMetadata\x12\x37\n\x0foutput_metadata\x18\x06 \x03(\x0b\x32\x1e.skytorch.torch.TensorMetadata\x1aK\n\x0bKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.skytorch.torch.AtenArgument:\x02\x38\x01\"p\n\x13\x45xecuteAtenResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x37\n\x0eoutput_tensors\x18\x03 \x03(\x0b\x32\x1f.skytorch.torch.TensorReference\"\xae\x01\n\x13UpdateTensorRequest\x12\x11\n\ttensor_id\x18\x01 \x01(\x04\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\r\n\x05shape\x18\x03 \x03(\x03\x12\r\n\x05\x64type\x18\x04 \x01(\t\x12\x0e\n\x06stride\x18\x05 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x06 \x01(\x03\x12\x30\n\x08metadata\x18\x07 \x01(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"\x89\x01\n\x11GetTensorResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\r\n\x05shape\x18\x04 \x03(\x03\x12\r\n\x05\x64type\x18\x05 \x01(\t\x12\x0e\n\x06stride\x18\x06 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x07 \x01(\x03\"|\n\x16\x45xecuteFunctionRequest\x12\x0c\n\x04\x61rgs\x18\x01 \x01(\x0c\x12\x0e\n\x06kwargs\x18\x02 \x01(\x0c\x12\x17\n\x0f\x63\x61llable_source\x18\x03 \x01(\t\x12\x15\n\rcallable_name\x18\x04 \x01(\t\x12\x14\n\x0cretain_model\x18\x05 \x01(\x08\"\xbd\x01\n\x10RemoteTensorInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nstorage_id\x18\x02 \x01(\x03\x12\r\n\x05shape\x18\x03 \x03(\x03\x12\r\n\x05\x64type\x18\x04 \x01(\t\x12\x0e\n\x06stride\x18\x05 \x03(\x03\x12\x16\n\x0estorage_offset\x18\x06 \x01(\x03\x12\x16\n\x0estorage_nbytes\x18\x07 \x01(\x03\x12\x13\n\x0b\x64\x65vice_type\x18\x08 \x01(\t\x12\x14\n\x0c\x64\x65vice_index\x18\t \x01(\x05\"\x86\x01\n\x17\x45xecuteFunctionResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x31\n\x07tensors\x18\x03 \x03(\x0b\x32 .skytorch.torch.RemoteTensorInfo\x12\x10\n\x08model_id\x18\x04 \x01(\x04\"2\n\x12\x45xecuteFunctionLog\x12\x0e\n\x06stream\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\"\x8d\x01\n\x14\x45xecuteFunctionEvent\x12\x31\n\x03log\x18\x01 \x01(\x0b\x32\".skytorch.torch.ExecuteFunctionLogH\x00\x12\x39\n\x06result\x18\x02 \x01(\x0b\x32\'.skytorch.torch.ExecuteFunctionResponseH\x00\x42\x07\n\x05\x65vent\"S\n\x16RegisterTensorsRequest\x12\x39\n\rregistrations\x18\x01 \x03(\x0b\x32\".skytorch.torch.TensorRegistration\";\n\x12TensorRegistration\x12\x12\n\nstorage_id\x18\x01 \x01(\x03\x12\x11\n\ttensor_id\x18\x02 \x01(\x04\"\xea\x01\n\x1b\x45xecuteModuleForwardRequest\x12\x10\n\x08model_id\x18\x01 \x01(\x04\x12\x13\n\x0bmodule_path\x18\x02 \x01(\t\x12\x18\n\x10input_tensor_ids\x18\x03 \x03(\x04\x12\x36\n\x0einput_metadata\x18\x04 \x03(\x0b\x32\x1e.skytorch.torch.TensorMetadata\x12\x19\n\x11output_tensor_ids\x18\x05 \x03(\x04\x12\x37\n\x0foutput_metadata\x18\x06 \x03(\x0b\x32\x1e.skytorch.torch.TensorMetadata\"\x80\x01\n\x1c\x45xecuteModuleForwardResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x38\n\x0eoutput_tensors\x18\x03 \x03(\x0b\x32 .skytorch.torch.RemoteTensorInfo\"\'\n\x13ReleaseModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\x04\"S\n\x19\x42\x61tchedExecuteAtenRequest\x12\x36\n\noperations\x18\x01 \x03(\x0b\x32\".skytorch.torch.ExecuteAtenRequest\"\x9b\x06\n\rStreamRequest\x12:\n\x0c\x65xecute_aten\x18\x01 \x01(\x0b\x32\".skytorch.torch.ExecuteAtenRequestH\x00\x12>\n\x0e\x64\x65lete_tensors\x18\x02 \x01(\x0b\x32$.skytorch.torch.DeleteTensorsRequestH\x00\x12\x38\n\x0b\x63opy_tensor\x18\x03 \x01(\x0b\x32!.skytorch.torch.CopyTensorRequestH\x00\x12<\n\rupdate_tensor\x18\x04 \x01(\x0b\x32#.skytorch.torch.UpdateTensorRequestH\x00\x12\x36\n\nget_tensor\x18\x05 \x01(\x0b\x32 .skytorch.torch.GetTensorRequestH\x00\x12\x42\n\x10register_tensors\x18\x06 \x01(\x0b\x32&.skytorch.torch.RegisterTensorsRequestH\x00\x12I\n\x14\x62\x61tched_execute_aten\x18\x07 \x01(\x0b\x32).skytorch.torch.BatchedExecuteAtenRequestH\x00\x12\x1a\n\x10raw_execute_aten\x18\x08 \x01(\x0cH\x00\x12\"\n\x18raw_batched_execute_aten\x18\t \x01(\x0cH\x00\x12\x36\n\nget_scalar\x18\n \x01(\x0b\x32 .skytorch.torch.GetScalarRequestH\x00\x12M\n\x16\x65xecute_module_forward\x18\x0e \x01(\x0b\x32+.skytorch.torch.ExecuteModuleForwardRequestH\x00\x12<\n\rrelease_model\x18\x0f \x01(\x0b\x32#.skytorch.torch.ReleaseModelRequestH\x00\x12\x14\n\x0c\x63hunk_number\x18\x0b \x01(\r\x12\x14\n\x0ctotal_chunks\x18\x0c \x01(\r\x12\x13\n\x0btotal_bytes\x18\r \x01(\x04\x42\t\n\x07request\"\x8a\x05\n\x0eStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12;\n\x0c\x65xecute_aten\x18\x03 \x01(\x0b\x32#.skytorch.torch.ExecuteAtenResponseH\x00\x12\x38\n\x0e\x64\x65lete_tensors\x18\x04 \x01(\x0b\x32\x1e.skytorch.torch.TensorResponseH\x00\x12\x35\n\x0b\x63opy_tensor\x18\x05 \x01(\x0b\x32\x1e.skytorch.torch.TensorResponseH\x00\x12\x37\n\rupdate_tensor\x18\x06 \x01(\x0b\x32\x1e.skytorch.torch.TensorResponseH\x00\x12\x37\n\nget_tensor\x18\x07 \x01(\x0b\x32!.skytorch.torch.GetTensorResponseH\x00\x12:\n\x10register_tensors\x18\x08 \x01(\x0b\x32\x1e.skytorch.torch.TensorResponseH\x00\x12\x37\n\nget_scalar\x18\t \x01(\x0b\x32!.skytorch.torch.GetScalarResponseH\x00\x12N\n\x16\x65xecute_module_forward\x18\x0e \x01(\x0b\x32,.skytorch.torch.ExecuteModuleForwardResponseH\x00\x12\x14\n\x0c\x63hunk_number\x18\n \x01(\r\x12\x14\n\x0ctotal_chunks\x18\x0b \x01(\r\x12\x19\n\x11server_backlog_ns\x18\x0c \x01(\x04\x12\x18\n\x10server_handle_ns\x18\r \x01(\x04\x42\n\n\x08response2\xe9\x04\n\x07Service\x12M\n\x0cUpdateTensor\x12\x1b.skytorch.torch.TensorChunk\x1a\x1e.skytorch.torch.TensorResponse(\x01\x12L\n\tGetTensor\x12 .skytorch.torch.GetTensorRequest\x1a\x1b.skytorch.torch.TensorChunk0\x01\x12O\n\nCopyTensor\x12!.skytorch.torch.CopyTensorRequest\x1a\x1e.skytorch.torch.TensorResponse\x12U\n\rDeleteTensors\x12$.skytorch.torch.DeleteTensorsRequest\x1a\x1e.skytorch.torch.TensorResponse\x12_\n\x14\x45xecuteAtenOperation\x12\".skytorch.torch.ExecuteAtenRequest\x1a#.skytorch.torch.ExecuteAtenResponse\x12U\n\x10StreamOperations\x12\x1d.skytorch.torch.StreamRequest\x1a\x1e.skytorch.torch.StreamResponse(\x01\x30\x01\x12\x61\n\x0f\x45xecuteFunction\x12&.skytorch.torch.ExecuteFunctionRequest\x1a$.skytorch.torch.ExecuteFunctionEvent0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'skytorch.torch.server.service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EXECUTEATENREQUEST_KWARGSENTRY']._loaded_options = None
  _globals['_EXECUTEATENREQUEST_KWARGSENTRY']._serialized_options = b'8\001'
  _globals['_DELETETENSORSREQUEST']._serialized_start=55
  _globals['_DELETETENSORSREQUEST']._serialized_end=97
  _globals['_TENSORMETADATA']._serialized_start=100
  _globals['_TENSORMETADATA']._serialized_end=304
  _globals['_TENSORCHUNK']._serialized_start=307
  _globals['_TENSORCHUNK']._serialized_end=538
  _globals['_TENSORRESPONSE']._serialized_start=540
  _globals['_TENSORRESPONSE']._serialized_end=590
  _globals['_GETSCALARREQUEST']._serialized_start=592
  _globals['_GETSCALARREQUEST']._serialized_end=686
  _globals['_GETSCALARRESPONSE']._serialized_start=688
  _globals['_GETSCALARRESPONSE']._serialized_end=782
  _globals['_CREATETENSORREQUEST']._serialized_start=785
  _globals['_CREATETENSORREQUEST']._serialized_end=994
  _globals['_GETTENSORREQUEST']._serialized_start=997
  _globals['_GETTENSORREQUEST']._serialized_end=1154
  _globals['_COPYTENSORREQUEST']._serialized_start=1157
  _globals['_COPYTENSORREQUEST']._serialized_end=1389
  _globals['_TENSORREFERENCE']._serialized_start=1391
  _globals['_TENSORREFERENCE']._serialized_end=1427
  _globals['_ATENARGUMENT']._serialized_start=1430
  _globals['_ATENARGUMENT']._serialized_end=1757
  _globals['_ATENARGUMENTLIST']._serialized_start=1759
  _globals['_ATENARGUMENTLIST']._serialized_end=1841
  _globals['_EXECUTEATENREQUEST']._serialized_start=1844
  _globals['_EXECUTEATENREQUEST']._serialized_end=2230
  _globals['_EXECUTEATENREQUEST_KWARGSENTRY']._serialized_start=2155
  _globals['_EXECUTEATENREQUEST_KWARGSENTRY']._serialized_end=2230
  _globals['_EXECUTEATENRESPONSE']._serialized_start=2232
  _globals['_EXECUTEATENRESPONSE']._serialized_end=2344
  _globals['_UPDATETENSORREQUEST']._serialized_start=2347
  _globals['_UPDATETENSORREQUEST']._serialized_end=2521
  _globals['_GETTENSORRESPONSE']._serialized_start=2524
  _globals['_GETTENSORRESPONSE']._serialized_end=2661
  _globals['_EXECUTEFUNCTIONREQUEST']._serialized_start=2663
  _globals['_EXECUTEFUNCTIONREQUEST']._serialized_end=2787
  _globals['_REMOTETENSORINFO']._serialized_start=2790
  _globals['_REMOTETENSORINFO']._serialized_end=2979
  _globals['_EXECUTEFUNCTIONRESPONSE']._serialized_start=2982
  _globals['_EXECUTEFUNCTIONRESPONSE']._serialized_end=3116
  _globals['_EXECUTEFUNCTIONLOG']._serialized_start=3118
  _globals['_EXECUTEFUNCTIONLOG']._serialized_end=3168
  _globals['_EXECUTEFUNCTIONEVENT']._serialized_start=3171
  _globals['_EXECUTEFUNCTIONEVENT']._serialized_end=3312
  _globals['_REGISTERTENSORSREQUEST']._serialized_start=3314
  _globals['_REGISTERTENSORSREQUEST']._serialized_end=3397
  _globals['_TENSORREGISTRATION']._serialized_start=3399
  _globals['_TENSORREGISTRATION']._serialized_end=3458
  _globals['_EXECUTEMODULEFORWARDREQUEST']._serialized_start=3461
  _globals['_EXECUTEMODULEFORWARDREQUEST']._serialized_end=3695
  _globals['_EXECUTEMODULEFORWARDRESPONSE']._serialized_start=3698
  _globals['_EXECUTEMODULEFORWARDRESPONSE']._serialized_end=3826
  _globals['_RELEASEMODELREQUEST']._serialized_start=3828
  _globals['_RELEASEMODELREQUEST']._serialized_end=3867
  _globals['_BATCHEDEXECUTEATENREQUEST']._serialized_start=3869
  _globals['_BATCHEDEXECUTEATENREQUEST']._serialized_end=3952
  _globals['_STREAMREQUEST']._serialized_start=3955
  _globals['_STREAMREQUEST']._serialized_end=4750
  _globals['_STREAMRESPONSE']._serialized_start=4753
  _globals['_STREAMRESPONSE']._serialized_end=5403
  _globals['_SERVICE']._serialized_start=5406
  _globals['_SERVICE']._serialized_end=6023
# @@protoc_insertion_point(module_scope)
