# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: skytorch/server/metrics/metrics.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'skytorch/server/metrics/metrics.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%skytorch/server/metrics/metrics.proto\x12\x13skytorch.metrics.v1\"\xeb\x01\n\x06Metric\x12\x0c\n\x04name\x18\x01 \x01(\t\x12-\n\x04type\x18\x02 \x01(\x0e\x32\x1f.skytorch.metrics.v1.MetricType\x12\r\n\x05value\x18\x03 \x01(\x01\x12\x0c\n\x04unit\x18\x04 \x01(\t\x12\x37\n\x06labels\x18\x05 \x03(\x0b\x32\'.skytorch.metrics.v1.Metric.LabelsEntry\x12\x11\n\ttimestamp\x18\x06 \x01(\x03\x12\x0c\n\x04help\x18\x07 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"b\n\x0fMetricsSnapshot\x12,\n\x07metrics\x18\x01 \x03(\x0b\x32\x1b.skytorch.metrics.v1.Metric\x12\x0e\n\x06source\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\"\xc0\x01\n\x11GetMetricsRequest\x12\x14\n\x0cmetric_names\x18\x01 \x03(\t\x12\x0f\n\x07sources\x18\x02 \x03(\t\x12O\n\rlabel_filters\x18\x03 \x03(\x0b\x32\x38.skytorch.metrics.v1.GetMetricsRequest.LabelFiltersEntry\x1a\x33\n\x11LabelFiltersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"M\n\x12GetMetricsResponse\x12\x37\n\tsnapshots\x18\x01 \x03(\x0b\x32$.skytorch.metrics.v1.MetricsSnapshot\"\xe0\x01\n\x14StreamMetricsRequest\x12\x18\n\x10interval_seconds\x18\x01 \x01(\x01\x12\x14\n\x0cmetric_names\x18\x02 \x03(\t\x12\x0f\n\x07sources\x18\x03 \x03(\t\x12R\n\rlabel_filters\x18\x04 \x03(\x0b\x32;.skytorch.metrics.v1.StreamMetricsRequest.LabelFiltersEntry\x1a\x33\n\x11LabelFiltersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01*@\n\nMetricType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05GAUGE\x10\x01\x12\x0b\n\x07\x43OUNTER\x10\x02\x12\r\n\tHISTOGRAM\x10\x03\x32\xcc\x01\n\x07Metrics\x12]\n\nGetMetrics\x12&.skytorch.metrics.v1.GetMetricsRequest\x1a\'.skytorch.metrics.v1.GetMetricsResponse\x12\x62\n\rStreamMetrics\x12).skytorch.metrics.v1.StreamMetricsRequest\x1a$.skytorch.metrics.v1.MetricsSnapshot0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'skytorch.server.metrics.metrics_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_METRIC_LABELSENTRY']._loaded_options = None
  _globals['_METRIC_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_GETMETRICSREQUEST_LABELFILTERSENTRY']._loaded_options = None
  _globals['_GETMETRICSREQUEST_LABELFILTERSENTRY']._serialized_options = b'8\001'
  _globals['_STREAMMETRICSREQUEST_LABELFILTERSENTRY']._loaded_options = None
  _globals['_STREAMMETRICSREQUEST_LABELFILTERSENTRY']._serialized_options = b'8\001'
  _globals['_METRICTYPE']._serialized_start=901
  _globals['_METRICTYPE']._serialized_end=965
  _globals['_METRIC']._serialized_start=63
  _globals['_METRIC']._serialized_end=298
  _globals['_METRIC_LABELSENTRY']._serialized_start=253
  _globals['_METRIC_LABELSENTRY']._serialized_end=298
  _globals['_METRICSSNAPSHOT']._serialized_start=300
  _globals['_METRICSSNAPSHOT']._serialized_end=398
  _globals['_GETMETRICSREQUEST']._serialized_start=401
  _globals['_GETMETRICSREQUEST']._serialized_end=593
  _globals['_GETMETRICSREQUEST_LABELFILTERSENTRY']._serialized_start=542
  _globals['_GETMETRICSREQUEST_LABELFILTERSENTRY']._serialized_end=593
  _globals['_GETMETRICSRESPONSE']._serialized_start=595
  _globals['_GETMETRICSRESPONSE']._serialized_end=672
  _globals['_STREAMMETRICSREQUEST']._serialized_start=675
  _globals['_STREAMMETRICSREQUEST']._serialized_end=899
  _globals['_STREAMMETRICSREQUEST_LABELFILTERSENTRY']._serialized_start=542
  _globals['_STREAMMETRICSREQUEST_LABELFILTERSENTRY']._serialized_end=593
  _globals['_METRICS']._serialized_start=968
  _globals['_METRICS']._serialized_end=1172
# @@protoc_insertion_point(module_scope)
